// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:io';
import 'package:flutter/services.dart';
import 'settings.dart';
import 'utils.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:nertc_core/nertc_core.dart';
import 'package:path_provider/path_provider.dart';

class CallPage extends StatefulWidget {
  final String cid;
  final int uid;

  CallPage({Key? key, required this.cid, required this.uid});

  @override
  _CallPageState createState() {
    return _CallPageState();
  }
}

class _CallPageState extends State<CallPage>
    with
        AppLoggerMixin,
        NERtcVideoRendererEventListener,
        NERtcChannelEventCallback,
        NERtcAudioMixingEventCallback,
        NERtcAudioEffectEventCallback,
        NERtcStatsEventCallback,
        NERtcDeviceEventCallback,
        NERtcLiveTaskCallback {
  late Settings _settings;
  NERtcEngine _engine = NERtcEngine.instance;
  List<_UserSession> _remoteSessions = [];
  _UserSession? _localSession;
  _UserSession? _localSubStreamSession;
  OverlayEntry? _overlayEntry;

  bool showControlPanel = false;
  bool isFullScreen = false;
  bool isDebugEnv = false;
  late Size screenSize;
  // Control 1
  bool isAudioEnabled = false;
  bool isVideoEnabled = false;
  bool isFrontCamera = true;
  bool isFrontCameraMirror = true;
  bool isSpeakerphoneOn = false;

  // Control 2
  bool isAudioDumpEnabled = false;
  bool isEarBackEnabled = false;
  bool isScreenRecordEnabled = false;

  // Control 3
  bool isLeaveChannel = false;
  bool isAudience = false;
  bool isAudioMixPlaying = false;
  bool isAudioEffectPlaying = false;

  // Control 4
  bool isMuteAudio = false;
  bool isMuteVideo = false;
  bool isMuteMic = false;
  bool isMuteSpeaker = false;
  bool isSubAllAudio = true;

  bool setMediaSever = false;
  int videoViewFitType = 0;

  // force to call sub and unsub
  bool forceSubAndUnsubVideo = false;
  final foregroundServiceChannel = const MethodChannel(
      'com.netease.nertc_core.nertc_core_example.foreground_service');

  @override
  void initState() {
    super.initState();
    print('start call: uid=${widget.uid}, cid=${widget.cid}');
    _initSettings().then((value) => _initRtcEngine());
    if (Platform.isAndroid) {
      foregroundServiceChannel.invokeMethod('startForegroundService');
    }
  }

  Future<void> _initSettings() async {
    _settings = await Settings.getInstance();
    isAudioEnabled = _settings.autoEnableAudio;
    isVideoEnabled = _settings.autoEnableVideo;
    isFrontCamera = _settings.frontFacingCamera;
    isFrontCameraMirror = _settings.frontFacingCameraMirror;
    videoViewFitType = _settings.videoViewFitType;
    updateLocalMirror();
  }

  void updateLocalMirror() {
    _localSession?.mirror.value = isFrontCamera && isFrontCameraMirror;
  }

  Future<void> updateSettings() async {
    isSpeakerphoneOn = await _engine.deviceManager.isSpeakerphoneOn();
    showControlPanel = true;
  }

  @override
  Widget build(BuildContext context) {
    screenSize = MediaQuery.of(context).size;

    return WillPopScope(
      child: GestureDetector(
        onTap: () {
          setState(() {
            showControlPanel = !showControlPanel;
          });
        },
        child: Scaffold(
          appBar: AppBar(
            automaticallyImplyLeading: false,
            centerTitle: true,
            title: Text(widget.cid),
          ),
          body: buildCallingWidget(context),
        ),
      ),
      // ignore: missing_return
      onWillPop: () {
        _requestPop();
        return Future.value(true);
      },
    );
  }

  Widget buildControlButton(VoidCallback onPressed, Widget child) {
    return ElevatedButton(
      onPressed: onPressed,
      child: child,
    );
  }

  Widget buildCallingWidget(BuildContext context) {
    return Stack(children: <Widget>[
      buildVideoViews(context),
      if (showControlPanel) buildControlPanel(context)
    ]);
  }

  Widget buildControlPanel(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.end,
      children: [
        buildControlPanel1(context),
        buildControlPanel2(context),
        buildControlPanel3(context),
        buildControlPanel4(context)
      ],
    );
  }

  Widget buildControlPanel4(BuildContext context) {
    List<Widget> children = [];
    children.add(Expanded(
        child: buildControlButton(
      () {
        bool muteAudio = !isMuteAudio;
        _engine.muteLocalAudioStream(muteAudio).then((value) {
          if (value == 0) {
            setState(() {
              isMuteAudio = muteAudio;
            });
          }
        });
      },
      Text(
        isMuteAudio ? '取消静音' : '静音',
        style: TextStyle(fontSize: 10),
      ),
    )));
    children.add(Expanded(
        child: buildControlButton(
      () {
        bool muteVideo = !isMuteVideo;
        _engine.muteLocalVideoStream(muteVideo).then((value) {
          if (value == 0) {
            setState(() {
              isMuteVideo = muteVideo;
            });
          }
        });
      },
      Text(
        isMuteVideo ? 'UnMuteVideo' : 'MuteVideo',
        style: TextStyle(fontSize: 10),
      ),
    )));
    children.add(Expanded(
        child: buildControlButton(
      () {
        bool muteMic = !isMuteMic;
        _engine.deviceManager.setRecordDeviceMute(muteMic).then((value) {
          if (value == 0) {
            setState(() {
              isMuteMic = muteMic;
            });
          }
        });
      },
      Text(
        isMuteMic ? 'UnMuteMic' : 'MuteMic',
        style: TextStyle(fontSize: 10),
      ),
    )));
    children.add(Expanded(
        child: buildControlButton(
      () {
        bool muteSpeaker = !isMuteSpeaker;
        _engine.deviceManager.setPlayoutDeviceMute(muteSpeaker).then((value) {
          if (value == 0) {
            setState(() {
              isMuteSpeaker = muteSpeaker;
            });
          }
        });
      },
      Text(
        isMuteSpeaker ? 'UnMuteSpeaker' : 'MuteSpeaker',
        style: TextStyle(fontSize: 10),
      ),
    )));
    children.add(Expanded(
        child: buildControlButton(
      () {
        bool subAllAudio = !isSubAllAudio;
        _engine.subscribeAllRemoteAudio(subAllAudio).then((value) {
          if (value == 0) {
            setState(() {
              isSubAllAudio = subAllAudio;
            });
          }
        });
      },
      Text(
        isSubAllAudio ? 'UnSubAllA' : 'SubAllA',
        style: TextStyle(fontSize: 10),
      ),
    )));
    return Container(
        height: 40,
        child: Row(
          children: children,
        ));
  }

  Widget buildControlPanel3(BuildContext context) {
    List<Widget> children = [];
    if (Platform.isIOS) {
      children.add(Expanded(
          child: buildControlButton(
        () {
          _requestPop();
        },
        Text(
          '结束通话',
          style: TextStyle(fontSize: 12),
        ),
      )));
    }
    children.add(Expanded(
        child: buildControlButton(
      () {
        bool leaveChannel = !isLeaveChannel;
        if (leaveChannel) {
          _engine.leaveChannel().then((value) {
            if (value == 0) {
              setState(() {
                _localSession = null;
                isLeaveChannel = leaveChannel;
              });
            }
          });
        } else {
          _initCallbacks()
              .then((value) => _initAudio())
              .then((value) => _initVideo())
              .then((value) => _initRenderer())
              .then((value) => _engine.joinChannel('', widget.cid, widget.uid))
              .then((value) {
            if (value == 0) {
              setState(() {
                isLeaveChannel = leaveChannel;
              });
            }
          });
        }
      },
      Text(
        isLeaveChannel ? '加入房间' : '离开房间',
        style: TextStyle(fontSize: 12),
      ),
    )));
    children.add(Expanded(
        child: buildControlButton(
      () {
        bool audience = !isAudience;
        _engine
            .setClientRole(
                audience ? NERtcUserRole.audience : NERtcUserRole.broadcaster)
            .then((value) {
          if (value == 0) {
            setState(() {
              isAudience = audience;
            });
          }
        });
      },
      Text(
        isAudience ? '切主播' : '切观众',
        style: TextStyle(fontSize: 12),
      ),
    )));

    children.add(Expanded(
        child: buildControlButton(
      () {
        bool playAudioMixing = !isAudioMixPlaying;
        if (playAudioMixing) {
          String path = _settings.audioMixingFileUrl.isNotEmpty
              ? _settings.audioMixingFileUrl
              : _settings.audioEffectFilePath;
          NERtcAudioMixingOptions options = NERtcAudioMixingOptions(
              path: path,
              sendEnabled: _settings.audioMixingSendEnabled,
              playbackEnabled: _settings.audioMixingPlayEnabled);
          _engine.audioMixingManager.startAudioMixing(options).then((value) {
            if (value == 0) {
              setState(() {
                isAudioMixPlaying = playAudioMixing;
              });
            }
          });
        } else {
          _engine.audioMixingManager.stopAudioMixing().then((value) {
            if (value == 0) {
              setState(() {
                isAudioMixPlaying = playAudioMixing;
              });
            }
          });
        }
      },
      Text(
        isAudioMixPlaying ? '结束伴音' : '开始伴音',
        style: TextStyle(fontSize: 12),
      ),
    )));
    if (Platform.isAndroid) {
      children.add(Expanded(
          child: buildControlButton(
        () {
          bool playAudioEffect = !isAudioEffectPlaying;
          if (playAudioEffect) {
            NERtcAudioEffectOptions options = NERtcAudioEffectOptions(
              path: Settings.defaultFileUrl,
              sendEnabled: _settings.audioEffectSendEnabled,
              playbackEnabled: _settings.audioEffectPlayEnabled,
            );
            _engine.audioEffectManager.playEffect(1, options).then((value) {
              if (value == 0) {
                setState(() {
                  isAudioEffectPlaying = playAudioEffect;
                });
              }
            });
          } else {
            _engine.audioEffectManager.stopEffect(1).then((value) {
              if (value == 0) {
                setState(() {
                  isAudioEffectPlaying = playAudioEffect;
                });
              }
            });
          }
        },
        Text(
          isAudioEffectPlaying ? '结束音效' : '开始音效',
          style: TextStyle(fontSize: 12),
        ),
      )));
    }
    return Container(
        height: 40,
        child: Row(
          children: children,
        ));
  }

  Widget buildControlPanel1(BuildContext context) {
    return Container(
        height: 40,
        child: Row(
          children: [
            Expanded(
                child: buildControlButton(
              () {
                bool audioEnabled = !isAudioEnabled;
                _engine.enableLocalAudio(audioEnabled).then((value) {
                  if (value == 0) {
                    setState(() {
                      isAudioEnabled = audioEnabled;
                    });
                  }
                });
              },
              Text(
                isAudioEnabled ? '关闭语音' : '打开语音',
                style: TextStyle(fontSize: 12),
              ),
            )),
            Expanded(
                child: buildControlButton(
              () {
                bool videoEnabled = !isVideoEnabled;
                _engine.enableLocalVideo(videoEnabled).then((value) {
                  if (value == 0) {
                    setState(() {
                      isVideoEnabled = videoEnabled;
                    });
                  }
                });
              },
              Text(
                isVideoEnabled ? '关闭视频' : '打开视频',
                style: TextStyle(fontSize: 12),
              ),
            )),
            Expanded(
                child: buildControlButton(
              () {
                _engine.deviceManager.switchCamera().then((value) {
                  if (value == 0) {
                    isFrontCamera = !isFrontCamera;
                    updateLocalMirror();
                  }
                });
              },
              Text(
                '切换摄像头',
                style: TextStyle(fontSize: 12),
              ),
            )),
            Expanded(
                child: buildControlButton(
              () {
                bool speakerphoneOn = !isSpeakerphoneOn;
                _engine.deviceManager
                    .setSpeakerphoneOn(speakerphoneOn)
                    .then((value) {
                  if (value == 0) {
                    setState(() {
                      isSpeakerphoneOn = speakerphoneOn;
                    });
                  }
                });
              },
              Text(
                isSpeakerphoneOn ? '关闭扬声器' : '打开扬声器',
                style: TextStyle(fontSize: 12),
              ),
            )),
          ],
        ));
  }

  Widget buildControlPanel2(BuildContext context) {
    List<Widget> children = [];
    children.add(Expanded(
        child: buildControlButton(
      () {
        bool audioDumpEnabled = !isAudioDumpEnabled;
        if (audioDumpEnabled) {
          _engine.startAudioDump().then((value) {
            if (value == 0) {
              setState(() {
                isAudioDumpEnabled = audioDumpEnabled;
              });
            }
          });
        } else {
          _engine.stopAudioDump().then((value) {
            if (value == 0) {
              setState(() {
                isAudioDumpEnabled = audioDumpEnabled;
              });
            }
          });
        }
      },
      Text(
        isAudioDumpEnabled ? '录音关' : '录音开',
        style: TextStyle(fontSize: 12),
      ),
    )));
    children.add(Expanded(
        child: buildControlButton(
      () {
        bool earBackEnabled = !isEarBackEnabled;
        _engine.deviceManager.enableEarback(earBackEnabled, 100).then((value) {
          if (value == 0) {
            setState(() {
              isEarBackEnabled = earBackEnabled;
            });
          }
        });
      },
      Text(
        isEarBackEnabled ? '耳返开' : '耳返关',
        style: TextStyle(fontSize: 12),
      ),
    )));
    if (Platform.isAndroid) {
      children.add(Expanded(
          child: buildControlButton(
        () {
          bool screenRecordEnabled = !isScreenRecordEnabled;
          if (screenRecordEnabled) {
            NERtcScreenConfig config = NERtcScreenConfig();
            config.contentPrefer = _settings.screenContentPrefer;
            _engine.startScreenCapture(config).then((value) {
              if (value == 0) {
                setState(() {
                  isScreenRecordEnabled = screenRecordEnabled;
                  _localSubStreamSession = _UserSession(widget.uid, true);
                });
              }
            });
          } else {
            _engine.stopScreenCapture().then((value) {
              if (value == 0) {
                setState(() {
                  isScreenRecordEnabled = screenRecordEnabled;
                  _localSubStreamSession = null;
                });
              }
            });
          }
        },
        Text(
          isScreenRecordEnabled ? '屏幕录制关' : '屏幕录制开',
          style: TextStyle(fontSize: 12),
        ),
      )));
    }
    children.add(Expanded(
        child: buildControlButton(
      () {
        _engine.uploadSdkInfo().then((value) {
          Fluttertoast.showToast(
              msg: 'upload sdk info result:$value',
              gravity: ToastGravity.CENTER);
        });
      },
      Text(
        '上传日志',
        style: TextStyle(fontSize: 12),
      ),
    )));
    return Container(
        height: 40,
        child: Row(
          children: children,
        ));
  }

  void _showOverlay(BuildContext context, _UserSession session,
      NERtcVideoRenderer? renderer) {
    if (_overlayEntry == null) {
      // 创建全屏视图
      Widget overlayWidget = GestureDetector(
          onTap: () {
            _hideOverlay();
            setState(() {
              isFullScreen = false;
            });
          },
          child: SafeArea(
              child: Scaffold(
            body: Stack(
              children: [
                NERtcVideoView(
                  uid: session.uid == widget.uid ? null : session.uid,
                  subStream: session.subStream,
                  mirrorListenable: session.mirror,
                  renderer: renderer,
                  rendererEventLister: this,
                ),
                Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    Text(
                      session.subStream
                          ? '${session.uid} @ Sub'
                          : '${session.uid}',
                      style: TextStyle(color: Colors.red, fontSize: 10),
                    )
                  ],
                )
              ],
            ),
          )));
      // 创建OverlayEntry
      _overlayEntry = OverlayEntry(builder: (BuildContext context) {
        return overlayWidget;
      });
      // 添加OverlayEntry到Overlay中
      Overlay.of(context)?.insert(_overlayEntry!);
    }
  }

  void _hideOverlay() {
    if (_overlayEntry != null) {
      _overlayEntry?.remove();
      _overlayEntry = null;
    }
  }

  Widget buildVideoViews(BuildContext context) {
    final sessions = [
      if (_localSession != null) _localSession!,
      if (_localSubStreamSession != null) _localSubStreamSession!,
      ..._remoteSessions,
    ];

    return OrientationBuilder(
      builder: (BuildContext context, Orientation orientation) {
        final isLandscape = orientation == Orientation.landscape;
        return GridView.builder(
          scrollDirection: isLandscape ? Axis.horizontal : Axis.vertical,
          gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
            crossAxisCount: isLandscape ? 1 : 3,
            childAspectRatio: isLandscape ? 16 / 9 : 9 / 16,
            crossAxisSpacing: 2.0,
            mainAxisSpacing: 2.0,
          ),
          itemCount: sessions.length,
          itemBuilder: (BuildContext context, int index) {
            return Stack(
              children: [
                GestureDetector(
                    onTap: () {
                      setState(() {
                        isFullScreen = true;
                      });
                      NERtcVideoRenderer? renderer;
                      void visit(Element element) {
                        if (element is StatefulElement &&
                            element.state is NERtcVideoViewState) {
                          renderer =
                              (element.state as NERtcVideoViewState).renderer;
                          return;
                        } else {
                          element.visitChildElements(visit);
                        }
                      }

                      context.visitChildElements(visit);
                      if (renderer != null) {
                        _showOverlay(context, sessions[index], renderer);
                      }
                    },
                    child: buildVideoView(context, sessions[index])),
              ],
            );
          },
        );
      },
    );
  }

  Widget buildVideoView(BuildContext context, _UserSession session) {
    return Container(
      child: Stack(
        children: [
          NERtcVideoView(
            uid: session.uid == widget.uid ? null : session.uid,
            subStream: session.subStream,
            mirrorListenable: session.mirror,
            rendererEventLister: this,
            fitType: videoViewFitType == 0
                ? NERtcVideoViewFitType.contain
                : NERtcVideoViewFitType.cover,
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              Text(
                session.subStream ? '${session.uid} @ Sub' : '${session.uid}',
                style: TextStyle(color: Colors.red, fontSize: 10),
              )
            ],
          )
        ],
      ),
    );
  }

  void _requestPop() {
    Navigator.pop(context);
  }

  Future<String?> getLogPath() async {
    Directory? directory;
    if (Platform.isIOS) {
      directory = await getApplicationDocumentsDirectory();
    } else if (Platform.isAndroid) {
      directory = await getExternalStorageDirectory();
    }
    if (directory != null) {
      return '${directory.path}/NERtcSDKTmp/log';
    }
    return null;
  }

  void _initRtcEngine() async {
    NERtcOptions options = NERtcOptions(
        // logDir: await getLogPath(),
        logLevel: NERtcLogLevel.info,
        audioAutoSubscribe: _settings.autoSubscribeAudio,
        videoAutoSubscribe: _settings.autoSubscribeVideo,
        disableFirstJoinUserCreateChannel:
            _settings.disableFirstJoinUserCreateChannel,
        serverRecordSpeaker: _settings.serverRecordSpeaker,
        serverRecordAudio: _settings.serverRecordAudio,
        serverRecordVideo: _settings.serverRecordVideo,
        serverRecordMode:
            NERtcServerRecordMode.values[_settings.serverRecordMode],
        videoSendMode: NERtcVideoSendMode.values[_settings.videoSendMode],
        videoEncodeMode:
            NERtcMediaCodecMode.values[_settings.videoEncodeMediaCodecMode],
        videoDecodeMode:
            NERtcMediaCodecMode.values[_settings.videoDecodeMediaCodecMode]);
    // 错误异常需要处理
    _engine
        .create(
            appKey: _settings.appKey,
            channelEventCallback: this,
            options: options)
        .then((value) => _initCallbacks())
        .then((value) => _initAudio())
        // .then((value) => _initCameraCaptureConfig())
        .then((value) => _initVideo())
        .then((value) => _initRenderer())
        .then((value) => _engine.joinChannel('', widget.cid, widget.uid))
        .catchError((e) {
      Fluttertoast.showToast(
          msg: 'catchError:' + e.toString(), gravity: ToastGravity.CENTER);
    });
  }

  Future<int?> _initCallbacks() async {
    _engine.setStatsEventCallback(this);
    _engine.audioMixingManager.setEventCallback(this);
    _engine.audioEffectManager.setEventCallback(this);
    _engine.deviceManager.setEventCallback(this);
    _engine.setLiveTaskEventCallback(this);
    _engine.setEventCallback(this);
    return 0;
  }

  Future<int?> _initAudio() async {
    if (Platform.isIOS) {
      await _engine.setAudioSessionOperationRestriction(
          NERtcAudioSessionOperationRestriction
              .values[_settings.audioSessionRestriction]);
    }
    await _engine.setAudioProfile(
        NERtcAudioProfile.values[_settings.audioProfile],
        NERtcAudioScenario.values[_settings.audioScenario]);
    return _engine.enableLocalAudio(isAudioEnabled);
  }

  Future<int?> _initCameraCaptureConfig() async {
    final config = NERtcCameraCaptureConfig.manual(640, 360);
    return _engine.setCameraCaptureConfig(config);
  }

  Future<int?> _initVideo() async {
    await _engine.enableLocalVideo(isVideoEnabled);
    await _engine.enableDualStreamMode(_settings.enableDualStreamMode);
    NERtcVideoConfig config = NERtcVideoConfig();
    config.videoProfile = _settings.videoProfile;
    config.frameRate = _settings.videoFrameRate;
    config.degradationPrefer = _settings.degradationPreference;
    config.frontCamera = _settings.frontFacingCamera;
    config.videoCropMode = _settings.videoCropMode;
    return _engine.setLocalVideoConfig(config);
  }

  Future<void> _initRenderer() async {
    setState(() {
      _localSession = _UserSession(widget.uid);
      updateLocalMirror();
    });
  }

  void _releaseRtcEngine() {
    _engine.release();
  }

  void _leaveChannel() async {
    await _engine.enableLocalVideo(false);
    await _engine.enableLocalAudio(false);
    await _engine.stopVideoPreview();
    _engine.removeStatsEventCallback(this);
    _localSession = null;
    _localSubStreamSession = null;
    _remoteSessions.clear();
    await _engine.leaveChannel();
  }

  @override
  void dispose() {
    _leaveChannel();
    _releaseRtcEngine();
    if (Platform.isAndroid) {
      foregroundServiceChannel.invokeMethod('cancelForegroundService');
    }
    super.dispose();
  }

  @override
  void onAddLiveStreamTask(String taskId, int errCode) {
    Fluttertoast.showToast(
        msg: 'onAddLiveStreamTask#$taskId ,errorCode:$errCode}',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onConnectionTypeChanged(int newConnectionType) {
    Fluttertoast.showToast(
        msg:
            'onConnectionTypeChanged#${Utils.connectionType2String(newConnectionType)}',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onDisconnect(int reason) {
    Fluttertoast.showToast(
        msg: 'onDisconnect#$reason', gravity: ToastGravity.CENTER);
  }

  @override
  void onFirstAudioDataReceived(int uid) {
    Fluttertoast.showToast(
        msg: 'onFirstAudioDataReceived#$uid', gravity: ToastGravity.CENTER);
  }

  @override
  void onFirstVideoDataReceived(int uid, int? streamType) {
    Fluttertoast.showToast(
        msg: 'onFirstVideoDataReceived#$uid,streamType:$streamType',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onLeaveChannel(int result) {
    Fluttertoast.showToast(
        msg: 'onLeaveChannel#$result', gravity: ToastGravity.CENTER);
  }

  @override
  void onUserAudioMute(int uid, bool muted) {
    Fluttertoast.showToast(
        msg: 'onUserAudioStart#uid:$uid, muted:$muted',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onUserAudioStart(int uid) {
    Fluttertoast.showToast(
        msg: 'onUserAudioStart#$uid', gravity: ToastGravity.CENTER);
  }

  @override
  void onUserAudioStop(int uid) {
    Fluttertoast.showToast(
        msg: 'onUserAudioStop#$uid', gravity: ToastGravity.CENTER);
  }

  @override
  void onUserLeave(
      int uid, int reason, NERtcUserLeaveExtraInfo? leaveExtraInfo) {
    Fluttertoast.showToast(
        msg:
            'onUserLeave#uid:$uid, reason:$reason,leaveExtraInfo:$leaveExtraInfo',
        gravity: ToastGravity.CENTER);
    for (_UserSession session in _remoteSessions.toList()) {
      if (session.uid == uid) {
        _remoteSessions.remove(session);
      }
    }
    setState(() {});
  }

  @override
  void onUserVideoMute(int uid, bool muted, int? streamType) {
    Fluttertoast.showToast(
        msg:
            'onUserVideoProfileUpdate#uid:$uid, muted:$muted,streamType:$streamType',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onUserVideoStart(int uid, int maxProfile) {
    Fluttertoast.showToast(
        msg:
            'onUserVideoStart#uid:$uid, maxProfile:${Utils.videoProfile2String(maxProfile)}',
        gravity: ToastGravity.CENTER);
    setupVideoView(uid, maxProfile, false);
  }

  Future<void> setupVideoView(int uid, int maxProfile, bool subStream) async {
    final session = _UserSession(uid, subStream);
    _remoteSessions.add(session);

    if (forceSubAndUnsubVideo || !_settings.autoSubscribeVideo) {
      if (subStream) {
        _engine.subscribeRemoteSubStreamVideo(uid, true);
      } else {
        _engine.subscribeRemoteVideoStream(
            uid, NERtcRemoteVideoStreamType.high, true);
      }
    }
    setState(() {});
  }

  Future<void> releaseVideoView(int uid, bool subStream) async {
    for (_UserSession session in _remoteSessions.toList()) {
      if (session.uid == uid && subStream == session.subStream) {
        _remoteSessions.remove(session);
        if (forceSubAndUnsubVideo || !_settings.autoSubscribeVideo) {
          if (!subStream) {
            _engine.subscribeRemoteVideoStream(
                uid, NERtcRemoteVideoStreamType.high, false);
          } else {
            _engine.subscribeRemoteSubStreamVideo(uid, false);
          }
        }
        setState(() {});
        break;
      }
    }
  }

  @override
  void onUserVideoStop(int uid) {
    Fluttertoast.showToast(
        msg: 'onUserVideoStop#$uid', gravity: ToastGravity.CENTER);
    releaseVideoView(uid, false);
  }

  @override
  void onJoinChannel(int result, int channelId, int elapsed, int uid) {
    Fluttertoast.showToast(
        msg:
            'onJoinChannel#$uid,channelId:$channelId,result:$result,elapsed:$elapsed',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onTakeSnapshotResult(int code, String path) {
    // TODO: implement onTakeSnapshotResult
    super.onTakeSnapshotResult(code, path);
  }

  @override
  void onRemoteVideoStats(List<NERtcVideoRecvStats> statsList) {
    Fluttertoast.showToast(
        msg: 'onRemoteVideoStats#$statsList', gravity: ToastGravity.CENTER);
  }

  @override
  void onLocalVideoStats(NERtcVideoSendStats stats) {
    Fluttertoast.showToast(
        msg: 'onLocalVideoStats#$stats', gravity: ToastGravity.CENTER);
  }

  @override
  void onRemoteAudioStats(List<NERtcAudioRecvStats> statsList) {
    Fluttertoast.showToast(
        msg: 'onRemoteAudioStats#$statsList', gravity: ToastGravity.CENTER);
  }

  @override
  void onLocalAudioStats(NERtcAudioSendStats stats) {
    Fluttertoast.showToast(
        msg: 'onLocalAudioStats#$stats', gravity: ToastGravity.CENTER);
  }

  @override
  void onRtcStats(NERtcStats stats) {
    Fluttertoast.showToast(
        msg: 'onRtcStats#$stats', gravity: ToastGravity.CENTER);
  }

  @override
  void onReJoinChannel(int result, int channelId) {
    Fluttertoast.showToast(
        msg: 'onReJoinChannel#$result,channelId:$channelId',
        gravity: ToastGravity.CENTER);
  }

  void onError(int code) {
    Fluttertoast.showToast(msg: 'onError#$code', gravity: ToastGravity.CENTER);
  }

  @override
  void onFirstAudioFrameDecoded(int uid) {
    Fluttertoast.showToast(
        msg: 'onFirstAudioFrameDecoded#$uid', gravity: ToastGravity.CENTER);
  }

  @override
  void onFirstVideoFrameDecoded(
      int uid, int width, int height, int? streamType) {
    Fluttertoast.showToast(
        msg: 'onFirstVideoFrameDecoded#uid:$uid, width:$width, height:$height',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onLocalAudioVolumeIndication(int volume, bool vadFlag) {
    Fluttertoast.showToast(
        msg: 'onLocalAudioVolumeIndication#$volume,vadFlag:$vadFlag',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onRemoteAudioVolumeIndication(
      List<NERtcAudioVolumeInfo> volumeList, int totalVolume) {
    Fluttertoast.showToast(
        msg:
            'onRemoteAudioVolumeIndication#$totalVolume,volumeList:$volumeList',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onWarning(int code) {
    Fluttertoast.showToast(
        msg: 'onWarning#$code', gravity: ToastGravity.CENTER);
  }

  @override
  void onNetworkQuality(List<NERtcNetworkQualityInfo> statsList) {
    // Fluttertoast.showToast(
    //     msg: 'onNetworkQuality:$statsList', gravity: ToastGravity.CENTER);
  }

  @override
  void onReconnectingStart() {
    Fluttertoast.showToast(
        msg: 'onReconnectingStart', gravity: ToastGravity.CENTER);
  }

  @override
  void onConnectionStateChanged(int state, int reason) {
    Fluttertoast.showToast(
        msg:
            'onConnectionStateChanged#state:${Utils.connectionState2String(state)}, '
            'reason:${Utils.connectionStateChangeReason2String(reason)}',
        gravity: ToastGravity.CENTER);
    switch (reason) {
      case NERtcConnectionStateChangeReason.leaveChannel:
        // 离开房间 ，通话已经结束了。建议优先关注 onLeaveChannel 回调
        break;

      case NERtcConnectionStateChangeReason.channelClosed:
        // 房间被关闭，通话已经结束了。建议优先关注onDisconnect 回调（code为NERtcConstants.ErrorCode.ENGINE_ERROR_ROOM_CLOSED）
        break;

      case NERtcConnectionStateChangeReason.serverKicked:
        // 被踢出房间，通话已经结束了。建议优先关注onDisconnect 回调（code为NERtcConstants.ErrorCode.ENGINE_ERROR_SERVER_KICKED）
        break;

      case NERtcConnectionStateChangeReason.timeout:
        // 网络超时，通话已经结束了。或关注onDisconnect 回调即可（其他code）
        break;

      case NERtcConnectionStateChangeReason.joinChannel:
        //开始加入房间
        break;

      case NERtcConnectionStateChangeReason.joinSucceed:
        //加入房间成功，与服务器成功建立连接。建议优先关注 onJoinChannel 回调即可
        break;

      case NERtcConnectionStateChangeReason.rejoinSucceed:
        // 重连成功，指在网络变化或超时后的重连成功通知
        break;

      case NERtcConnectionStateChangeReason.signalDisconnected:
        // 网络超时，指与媒体服务器暂时失去连接，后续SDK 内部会自动重连
        break;

      case NERtcConnectionStateChangeReason.mediaConnectionDisconnected:
        // 网络超时，指与信令服务器暂时失去连接，后续SDK 内部会自动重连
        break;

      case NERtcConnectionStateChangeReason.requestChannelFailed:
        //加入房间获取房间信息这一步就失败。建议优先关注 onJoinChannel 回调即可
        break;

      case NERtcConnectionStateChangeReason.joinChannelFailed:
        //加入房间时与服务器建立长连失败。建议优先关注 onJoinChannel 回调即可
        break;
    }
  }

  @override
  void onLiveStreamState(String taskId, String pushUrl, int liveState) {
    Fluttertoast.showToast(
        msg:
            'onLiveStreamState#taskId:$taskId, liveState:${Utils.liveStreamState2String(liveState)}',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onAudioDeviceChanged(int selected) {
    String audioDevice;
    switch (selected) {
      case NERtcAudioDevice.earpiece:
        audioDevice = "听筒";
        break;
      case NERtcAudioDevice.bluetoothHeadset:
        audioDevice = "蓝牙耳机";
        break;
      case NERtcAudioDevice.speakerPhone:
        audioDevice = "扬声器";
        break;
      case NERtcAudioDevice.wiredHeadset:
        audioDevice = "有线耳机";
        break;
      default:
        audioDevice = "EARPIECE";
        break;
    }
  }

  @override
  void onAudioDeviceStateChange(int deviceType, int deviceState) {
    Fluttertoast.showToast(
        msg:
            'onAudioDeviceStateChange#deviceType:${Utils.audioDeviceType2String(deviceType)}, '
            'deviceState:${Utils.audioDeviceState2String(deviceState)}',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onVideoDeviceStateChange(int deviceType, int deviceState) {
    Fluttertoast.showToast(
        msg:
            'onVideoDeviceStateChange#${Utils.videoDeviceState2String(deviceState)}',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onCameraExposureChanged(CGPoint exposurePoint) {
    print('onCameraExposureChanged#');
    //  Fluttertoast.showToast(
    //     msg: 'onCameraExposureChanged#$exposurePoint',
    //     gravity: ToastGravity.CENTER);
  }

  @override
  void onCameraFocusChanged(CGPoint focusPoint) {
    Fluttertoast.showToast(
        msg: 'onCameraFocusChanged#$focusPoint', gravity: ToastGravity.CENTER);
  }

  @override
  void onClientRoleChange(int oldRole, int newRole) {
    Fluttertoast.showToast(
        msg: 'onClientRoleChange#oldRole:$oldRole, newRole:$newRole',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onUserSubStreamVideoStart(int uid, int maxProfile) {
    Fluttertoast.showToast(
        msg:
            'onUserSubStreamVideoStart#uid:$uid, maxProfile:${Utils.videoProfile2String(maxProfile)}',
        gravity: ToastGravity.CENTER);
    setupVideoView(uid, maxProfile, true);
  }

  @override
  void onUserSubStreamVideoStop(int uid) {
    Fluttertoast.showToast(
        msg: 'onUserSubStreamVideoStop#$uid', gravity: ToastGravity.CENTER);
    releaseVideoView(uid, true);
  }

  @override
  void onAudioHasHowling() {
    Fluttertoast.showToast(
        msg: 'onAudioHasHowling', gravity: ToastGravity.CENTER);
  }

  @override
  void onRecvSEIMsg(int userID, String seiMsg) {
    Fluttertoast.showToast(
        msg: 'onReceiveSEIMsg:userID:$userID,seiMsg:$seiMsg',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onAudioRecording(int code, String filePath) {
    Fluttertoast.showToast(
        msg: 'onReceiveSEIMsg:code:$code,filePath:$filePath',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onMediaRelayReceiveEvent(int event, int code, String channelName) {}
  @override
  void onMediaRelayStatesChange(int state, String channelName) {}
  @override
  void onLocalPublishFallbackToAudioOnly(bool isFallback, int streamType) {}
  @override
  void onRemoteSubscribeFallbackToAudioOnly(
      int uid, bool isFallback, int streamType) {}
  @override
  void onAudioEffectFinished(int effectId) {
    Fluttertoast.showToast(
        msg: 'onAudioEffectFinished#$effectId', gravity: ToastGravity.CENTER);
    setState(() {
      isAudioEffectPlaying = false;
    });
  }

  @override
  void onAudioMixingStateChanged(int reason) {
    Fluttertoast.showToast(
        msg: 'onAudioMixingStateChanged#$reason', gravity: ToastGravity.CENTER);
    setState(() {
      isAudioMixPlaying = false;
    });
  }

  @override
  void onAudioMixingTimestampUpdate(int timestampMs) {}
  @override
  void onUserSubStreamAudioMute(int uid, bool muted) {
    Fluttertoast.showToast(
        msg: 'onUserSubStreamAudioMute#$uid $muted',
        gravity: ToastGravity.CENTER);
    setState(() {
      //...
    });
  }

  @override
  void onUserSubStreamAudioStart(int uid) {
    Fluttertoast.showToast(
        msg: 'onUserSubStreamAudioStart#$uid ', gravity: ToastGravity.CENTER);
  }

  @override
  void onUserSubStreamAudioStop(int uid) {
    Fluttertoast.showToast(
        msg: 'onUserSubStreamAudioStop#$uid ', gravity: ToastGravity.CENTER);
  }

  @override
  void onVirtualBackgroundSourceEnabled(bool enabled, int reason) {
    Fluttertoast.showToast(
        msg: 'onVirtualBackgroundSourceEnabled#$enabled, reason:$reason ',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onAudioEffectTimestampUpdate(int id, int timestampMs) {
    Fluttertoast.showToast(
        msg: 'onAudioEffectTimestampUpdate#$id, timestampMs:$timestampMs',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onLocalVideoWatermarkState(int videoStreamType, int state) {
    Fluttertoast.showToast(
        msg:
            'onLocalVideoWatermarkState#$state, videoStreamType:$videoStreamType',
        gravity: ToastGravity.CENTER);
  }

  @override
  void onUserJoined(int uid, NERtcUserJoinExtraInfo? joinExtraInfo) {
    Fluttertoast.showToast(
        msg: 'onUserJoined#$uid ,$joinExtraInfo', gravity: ToastGravity.CENTER);
    setState(() {});
  }

  @override
  void onFirstFrameRendered(int uid) {
    logger.i(
      'onFirstFrameRendered#uid:$uid',
    );
  }

  @override
  void onFrameResolutionChanged(int uid, int width, int height, int rotation) {
    logger.i(
        'onFrameResolutionChanged#uid:$uid, width:$width, height:$height, rotation:$rotation');
  }
}

class _UserSession {
  final int uid;
  final bool subStream;

  _UserSession(this.uid, [this.subStream = false]);

  ValueNotifier<bool>? _mirror;
  ValueNotifier<bool> get mirror {
    _mirror ??= ValueNotifier<bool>(false);
    return _mirror!;
  }
}
