// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nertc;

class _NERtcEngineImpl extends NERtcEngine
    with
        _SDKLoggerMixin,
        LoggingApi,
        NERtcChannelEventSink,
        NERtcDeviceEventSink,
        NERtcAudioEffectEventSink,
        NERtcAudioMixingEventSink,
        NERtcStatsEventSink,
        NERtcLiveStreamEventSink {
  late _NERtcDeviceManagerImpl _deviceManager = _NERtcDeviceManagerImpl();
  final _NERtcAudioEffectMgrImpl _audioEffectMgr = _NERtcAudioEffectMgrImpl();
  final _NERtcAudioMixingMgrImpl _audioMixingMgr = _NERtcAudioMixingMgrImpl();
  final _rtcStatsPlatform = NERtcStatsPlatform.instance;
  final _devicePlatform = NERtcDevicePlatform.instance;
  final _audioEffectPlatform = NERtcAudioEffectPlatform.instance;
  final _audioMixingPlatform = NERtcAudioMixingPlatform.instance;
  final _liveTaskPlatform = NERtcLiveStreamPlatform.instance;

  /// 获取设备管理模块
  NERtcDeviceManager get deviceManager => _deviceManager;
  NERtcAudioEffectManager get audioEffectManager => _audioEffectMgr;
  NERtcAudioMixingManager get audioMixingManager => _audioMixingMgr;

  final _eventCallbacks = <NERtcChannelEventCallback>{};
  final _statsCallbacks = <NERtcStatsEventCallback>{};
  final _liveTaskCallback = <NERtcLiveTaskCallback>{};
  final _platform = NERtcEnginePlatform.instance;

  _NERtcEngineImpl() {
    _platform.addRtcChannelListener(this);
    _rtcStatsPlatform.registerStatsListener(this);
    _devicePlatform.addRtcDeviceListener(this);
    _audioEffectPlatform.addRtcAudioEffectListener(this);
    _audioMixingPlatform.addRtcAudioMixingListener(this);
    _liveTaskPlatform.registerLiveStreamListener(this);
  }

  VoidCallback? onDispose;

  @override
  String get logTag => 'EngineApi';

  Future<int> create(
      {required String appKey,
      required NERtcChannelEventCallback channelEventCallback,
      NERtcOptions? options}) async {
    await _LogService().init(appKey, options?.logLevel, options?.logDir);
    _initialized = true;
    apiLogger
        .i('createEngine#arg: appKey:$appKey, options:${options.toString()}');
    int reply =
        await wrapper('createEngine', _platform.create(appKey, options));
    return reply;
  }

  Future<NERtcVersion> version() async {
    return await wrapper('version', _platform.version());
  }

  Future<List<String?>?> checkPermission() async {
    return await wrapper('checkPermission', _platform.checkPermission());
  }

  Future<int> setParameters(NERtcParameters params) async {
    Map<String, Object> parameters = Map<String, Object>();
    parameters = params._parameters;
    int reply = await wrapper(
        'setParameters', _platform.setParameters(parameters), parameters);
    return reply;
  }

  void setEventCallback(NERtcChannelEventCallback callback) {
    _eventCallbacks.add(callback);
  }

  void removeEventCallback(NERtcChannelEventCallback callback) {
    _eventCallbacks.remove(callback);
  }

  void setStatsEventCallback(NERtcStatsEventCallback callback) {
    _statsCallbacks.add(callback);
    _platform.setStatsEventCallback();
    apiLogger.i('setStatsEventCallback');
    _rtcStatsPlatform.registerStatsListener(this);
  }

  void removeStatsEventCallback(NERtcStatsEventCallback callback) {
    _statsCallbacks.remove(callback);
  }

  void setLiveTaskEventCallback(NERtcLiveTaskCallback callback) {
    _liveTaskCallback.add(callback);
  }

  void removeLiveTaskEventCallback(NERtcLiveTaskCallback callback) {
    _liveTaskCallback.remove(callback);
  }

  void dispose() {
    commonLogger.i('dispose');
    _eventCallbacks.clear();
    _statsCallbacks.clear();
    _liveTaskCallback.clear();

    _platform.removeRtcChannelListener(this);
    _rtcStatsPlatform.unregisterStatsListener(this);
    _liveTaskPlatform.unregisterLiveStreamListener(this);
    _devicePlatform.removeRtcDeviceListener(this);
    _audioEffectPlatform.removeRtcAudioEffectListener(this);
    _audioMixingPlatform.removeRtcAudioMixingListener(this);

    onDispose?.call();
    onDispose = null;
  }

  @override
  void onJoinChannel(int result, int channelID, int elapsed, int uid) {
    commonLogger.i(
        'onJoinChannel: result:$result,channelId:$channelID,elapsed:$elapsed,uid:$uid');
    for (var callback in _eventCallbacks.copy()) {
      callback.onJoinChannel(result, channelID, elapsed, uid);
    }
  }

  @override
  void onLeaveChannel(int result) {
    commonLogger.i('onLeaveChannel: result:$result');
    for (var callback in _eventCallbacks.copy()) {
      callback.onLeaveChannel(result);
    }
  }

  @override
  void onUserJoined(UserJoinedEvent event) {
    if (!_initialized) return;
    final uid = event.uid;
    final joinExtraInfo = event.joinExtraInfo;
    commonLogger.i(
        'onUserJoined: uid:$uid,joinExtraInfo: {customInfo:${joinExtraInfo?.customInfo}}');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserJoined(uid, joinExtraInfo);
    }
  }

  @override
  void onUserLeave(UserLeaveEvent event) {
    if (!_initialized) return;
    final uid = event.uid;
    final reason = event.reason;
    final leaveExtraInfo = event.leaveExtraInfo;
    commonLogger.i(
        'onUserLeave: uid:$uid,reason:$reason,leaveExtraInfo:{customInfo:${leaveExtraInfo?.customInfo}}');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserLeave(uid, reason, leaveExtraInfo);
    }
  }

  @override
  void onDisconnect(int reason) {
    if (!_initialized) return;
    commonLogger.i('onDisconnect: reason:$reason');
    for (var callback in _eventCallbacks.copy()) {
      callback.onDisconnect(reason);
    }
  }

  @override
  void onUserAudioStart(int uid) {
    if (!_initialized) return;
    commonLogger.i('onUserAudioStart: uid:$uid');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserAudioStart(uid);
    }
  }

  @override
  void onUserAudioStop(int uid) {
    if (!_initialized) return;
    commonLogger.i('onUserAudioStop: uid:$uid');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserAudioStop(uid);
    }
  }

  @override
  void onUserSubStreamAudioStart(int uid) {
    if (!_initialized) return;
    commonLogger.i('onUserSubStreamAudioStart: uid:$uid');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserSubStreamAudioStart(uid);
    }
  }

  @override
  void onUserSubStreamAudioStop(int uid) {
    if (!_initialized) return;
    commonLogger.i('onUserSubStreamAudioStop: uid:$uid');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserSubStreamAudioStop(uid);
    }
  }

  @override
  void onUserVideoStart(int uid, int maxProfile) {
    if (!_initialized) return;
    commonLogger.i('onUserVideoStart: uid:$uid, maxProfile: $maxProfile');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserVideoStart(uid, maxProfile);
    }
  }

  @override
  void onUserVideoStop(int uid) {
    if (!_initialized) return;
    commonLogger.i('onUserVideoStop: uid:$uid');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserVideoStop(uid);
    }
  }

  @override
  void onUserAudioMute(int uid, bool muted) {
    if (!_initialized) return;
    commonLogger.i('onUserAudioMute: uid:$uid, muted:$muted');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserAudioMute(uid, muted);
    }
  }

  @override
  void onUserVideoMute(UserVideoMuteEvent event) {
    if (!_initialized) return;
    final uid = event.uid;
    final muted = event.muted;
    final streamType = event.streamType;
    commonLogger
        .i('onUserVideoMute: uid:$uid, muted:$muted, streamType:$streamType');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserVideoMute(uid, muted, streamType);
    }
  }

  @override
  void onFirstAudioDataReceived(int uid) {
    if (!_initialized) return;
    commonLogger.i('onFirstAudioDataReceived: uid:$uid');
    for (var callback in _eventCallbacks.copy()) {
      callback.onFirstAudioDataReceived(uid);
    }
  }

  @override
  void onFirstAudioFrameDecoded(int uid) {
    if (!_initialized) return;
    commonLogger.i('onFirstAudioFrameDecoded: uid:$uid');
    for (var callback in _eventCallbacks.copy()) {
      callback.onFirstAudioFrameDecoded(uid);
    }
  }

  @override
  void onFirstVideoDataReceived(FirstVideoDataReceivedEvent event) {
    if (!_initialized) return;
    final uid = event.uid;
    final streamType = event.streamType;
    commonLogger
        .i('onFirstVideoDataReceived: uid :$uid,streamType:$streamType');
    for (var callback in _eventCallbacks.copy()) {
      callback.onFirstVideoDataReceived(uid, streamType);
    }
  }

  @override
  void onFirstVideoFrameDecoded(FirstVideoFrameDecodedEvent event) {
    if (!_initialized) return;
    final uid = event.uid;
    final width = event.width;
    final height = event.height;
    final streamType = event.streamType;
    commonLogger.i(
        'onFirstVideoFrameDecoded: uid:$uid,width:$width,height:$height,streamType:$streamType');
    for (var callback in _eventCallbacks.copy()) {
      callback.onFirstVideoFrameDecoded(uid, width, height, streamType);
    }
  }

  @override
  void onUserSubStreamAudioMute(int uid, bool muted) {
    if (!_initialized) return;
    commonLogger.i('onUserSubStreamAudioMute: uid:$uid,muted:$muted');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserSubStreamAudioMute(uid, muted);
    }
  }

  @override
  void onVirtualBackgroundSourceEnabled(
      VirtualBackgroundSourceEnabledEvent event) {
    if (!_initialized) return;
    final enabled = event.enabled;
    final reason = event.reason;
    commonLogger
        .i('onVirtualBackgroundSourceEnabled: enabled:$enabled,reason:$reason');
    for (var callback in _eventCallbacks.copy()) {
      callback.onVirtualBackgroundSourceEnabled(enabled, reason);
    }
  }

  @override
  void onConnectionTypeChanged(int newConnectionType) {
    if (!_initialized) return;
    commonLogger
        .i('onConnectionTypeChanged: newConnectionType:$newConnectionType');
    for (var callback in _eventCallbacks.copy()) {
      callback.onConnectionTypeChanged(newConnectionType);
    }
  }

  @override
  void onReconnectingStart() {
    if (!_initialized) return;
    commonLogger.i('onReconnectingStart');
    for (var callback in _eventCallbacks.copy()) {
      callback.onReconnectingStart();
    }
  }

  @override
  void onReJoinChannel(int result, int channelId) {
    if (!_initialized) return;
    commonLogger.i('onReJoinChannel: result:$result, channelId:$channelId');
    for (var callback in _eventCallbacks.copy()) {
      callback.onReJoinChannel(result, channelId);
    }
  }

  @override
  void onConnectionStateChanged(int state, int reason) {
    if (!_initialized) return;
    commonLogger.i('onConnectionStateChanged: state:$state, reason:$reason');
    for (var callback in _eventCallbacks.copy()) {
      callback.onConnectionStateChanged(state, reason);
    }
  }

  @override
  void onLocalAudioVolumeIndication(int volume, bool vadFlag) {
    if (!_initialized) return;
    for (var callback in _eventCallbacks.copy()) {
      callback.onLocalAudioVolumeIndication(volume, vadFlag);
    }
  }

  @override
  void onRemoteAudioVolumeIndication(RemoteAudioVolumeIndicationEvent event) {
    if (!_initialized || event.volumeList == null) return;
    final volumeList = event.volumeList!
        .where((element) => element != null)
        .map((e) => NERtcAudioVolumeInfo(e!.uid, e.volume, e.subStreamVolume))
        .toList();
    final totalVolume = event.totalVolume;

    for (var callback in _eventCallbacks.copy()) {
      callback.onRemoteAudioVolumeIndication(volumeList, totalVolume);
    }
  }

  @override
  void onClientRoleChange(int oldRole, int newRole) {
    if (!_initialized) return;
    commonLogger.i('onClientRoleChange: oldRole:$oldRole, newRole:$newRole');
    for (var callback in _eventCallbacks.copy()) {
      callback.onClientRoleChange(oldRole, newRole);
    }
  }

  @override
  void onError(int code) {
    if (!_initialized) return;
    commonLogger.i('onError: code:$code');
    for (var callback in _eventCallbacks.copy()) {
      callback.onError(code);
    }
  }

  @override
  void onLiveStreamState(String taskId, String pushUrl, int liveState) {
    if (!_initialized) return;
    commonLogger.i(
        'onLiveStreamState: taskId:$taskId, pushUrl:$pushUrl, liveState:$liveState');
    for (var callback in _eventCallbacks.copy()) {
      callback.onLiveStreamState(taskId, pushUrl, liveState);
    }
  }

  @override
  void onWarning(int code) {
    if (!_initialized) return;
    commonLogger.i('onWarning: code:$code');
    for (var callback in _eventCallbacks.copy()) {
      callback.onWarning(code);
    }
  }

  @override
  void onAudioHasHowling() {
    if (!_initialized) return;
    commonLogger.i('onAudioHasHowling');
    for (var callback in _eventCallbacks.copy()) {
      callback.onAudioHasHowling();
    }
  }

  @override
  void onUserSubStreamVideoStart(int uid, int maxProfile) {
    if (!_initialized) return;
    commonLogger
        .i('onUserSubStreamVideoStart: uid:$uid, maxProfile:$maxProfile');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserSubStreamVideoStart(uid, maxProfile);
    }
  }

  @override
  void onUserSubStreamVideoStop(int uid) {
    if (!_initialized) return;
    commonLogger.i('onUserSubStreamVideoStop: uid:$uid');
    for (var callback in _eventCallbacks.copy()) {
      callback.onUserSubStreamVideoStop(uid);
    }
  }

  @override
  void onRecvSEIMsg(int userID, String seiMsg) {
    if (!_initialized) return;
    commonLogger.i('onRecvSEIMsg: userID:$userID, seiMsg:$seiMsg');
    for (var callback in _eventCallbacks.copy()) {
      callback.onRecvSEIMsg(userID, seiMsg);
    }
  }

  @override
  void onAudioRecording(int code, String filePath) {
    if (!_initialized) return;
    commonLogger.i('onAudioRecording: code:$code, filePath:$filePath');
    for (var callback in _eventCallbacks.copy()) {
      callback.onAudioRecording(code, filePath);
    }
  }

  @override
  void onMediaRelayStatesChange(int state, String channelName) {
    if (!_initialized) return;
    commonLogger
        .i('onMediaRelayStatesChange: state:$state, channelName:$channelName');
    for (var callback in _eventCallbacks.copy()) {
      callback.onMediaRelayStatesChange(state, channelName);
    }
  }

  @override
  void onLocalPublishFallbackToAudioOnly(bool isFallback, int streamType) {
    if (!_initialized) return;
    commonLogger.i(
        'onLocalPublishFallbackToAudioOnly: isFallback:$isFallback, streamType:$streamType');
    for (var callback in _eventCallbacks.copy()) {
      callback.onLocalPublishFallbackToAudioOnly(isFallback, streamType);
    }
  }

  @override
  void onMediaRelayReceiveEvent(int event, int code, String channelName) {
    if (!_initialized) return;
    commonLogger.i(
        'onMediaRelayReceiveEvent: event:$event, code:$code,channelName:$channelName');
    for (var callback in _eventCallbacks.copy()) {
      callback.onMediaRelayReceiveEvent(event, code, channelName);
    }
  }

  @override
  void onRemoteSubscribeFallbackToAudioOnly(
      int uid, bool isFallback, int streamType) {
    if (!_initialized) return;
    commonLogger.i(
        'onRemoteSubscribeFallbackToAudioOnly: uid:$uid, isFallback:$isFallback,streamType:$streamType');
    for (var callback in _eventCallbacks.copy()) {
      callback.onRemoteSubscribeFallbackToAudioOnly(
          uid, isFallback, streamType);
    }
  }

  @override
  void onLocalVideoWatermarkState(int videoStreamType, int state) {
    if (!_initialized) return;
    commonLogger.i(
        'onLocalVideoWatermarkState: videoStreamType:$videoStreamType, state:$state');
    for (var callback in _eventCallbacks.copy()) {
      callback.onLocalVideoWatermarkState(videoStreamType, state);
    }
  }

  @override
  void onAudioDeviceChanged(int selected) {
    if (!_initialized) return;
    commonLogger.i('onAudioDeviceChanged: selected:$selected');
    _deviceManager.deviceEventCallbacks.forEach((element) {
      element.onAudioDeviceChanged(selected);
    });
  }

  @override
  void onAudioDeviceStateChange(int deviceType, int deviceState) {
    if (!_initialized) return;
    commonLogger.i(
        'onAudioDeviceStateChange: deviceType:$deviceType, deviceState:$deviceState');
    _deviceManager.deviceEventCallbacks.forEach((element) {
      element.onAudioDeviceStateChange(deviceType, deviceState);
    });
  }

  @override
  void onVideoDeviceStateChange(int deviceType, int deviceState) {
    if (!_initialized) return;
    commonLogger.i('onVideoDeviceStateChange: deviceState:$deviceState');
    _deviceManager.deviceEventCallbacks.forEach((element) {
      element.onVideoDeviceStateChange(deviceType, deviceState);
    });
  }

  @override
  void onCameraExposureChanged(CGPoint exposurePoint) {
    if (!_initialized) return;
    _deviceManager.deviceEventCallbacks.forEach((element) {
      element.onCameraExposureChanged(exposurePoint);
    });
  }

  @override
  void onCameraFocusChanged(CGPoint focusPoint) {
    if (!_initialized) return;
    commonLogger.i('onCameraFocusChanged: focusPoint:$focusPoint');
    _deviceManager.deviceEventCallbacks.forEach((element) {
      element.onCameraFocusChanged(focusPoint);
    });
  }

  @override
  Future<int> release() async {
    _eventCallbacks.clear();
    _statsCallbacks.clear();
    _liveTaskCallback.clear();
    _initialized = false;
    return await wrapper('release', _platform.release());
  }

  @override
  Future<int> joinChannel(String? token, String channelName, int uid,
      [NERtcJoinChannelOptions? channelOptions]) async {
    apiLogger.i(
        'joinChannel#arg{token:$token,:channelName:$channelName,uid:$uid,channelOptions:${channelOptions.toString()}}');
    int reply = await wrapper('joinChannel',
        _platform.joinChannel(token, channelName, uid, channelOptions));
    return reply;
  }

  @override
  Future<int> leaveChannel() async {
    int reply = await wrapper('leaveChannel', _platform.leaveChannel());
    return reply;
  }

  @override
  Future<int> updatePermissionKey(String key) async {
    apiLogger.i('updatePermissionKey#arg{key:$key}');
    int reply = await wrapper(
        'updatePermissionKey', _platform.updatePermissionKey(key));
    return reply;
  }

  @override
  Future<int> enableLocalAudio(bool enable) async {
    apiLogger.i('enableLocalAudio#arg{enable:$enable}');
    int reply =
        await wrapper('enableLocalAudio', _platform.enableLocalAudio(enable));
    return reply;
  }

  @override
  Future<int> subscribeRemoteAudio(int uid, bool subscribe) async {
    apiLogger.i('subscribeRemoteAudio#arg{uid:$uid,subscribe:$subscribe}');
    int reply = await wrapper(
        'subscribeRemoteAudio', _platform.subscribeRemoteAudio(uid, subscribe));
    return reply;
  }

  @override
  Future<int> subscribeAllRemoteAudio(bool subscribe) async {
    apiLogger.i('subscribeAllRemoteAudio#arg{subscribe:$subscribe}');
    int reply = await wrapper('subscribeAllRemoteAudio',
        _platform.subscribeAllRemoteAudio(subscribe));
    return reply;
  }

  @override
  Future<int> enableLocalVideo(bool enable,
      {int streamType = NERtcVideoStreamType.main}) async {
    apiLogger.i('enableLocalVideo#arg{enable:$enable,streamType:$streamType}');
    int reply = await wrapper(
        'enableLocalVideo', _platform.enableLocalVideo(enable, streamType));
    return reply;
  }

  @override
  Future<int> muteLocalAudioStream(bool mute) async {
    apiLogger.i('muteLocalAudioStream#arg{mute:$mute}');
    int reply = await wrapper(
        'muteLocalAudioStream', _platform.muteLocalAudioStream(mute));
    return reply;
  }

  @override
  Future<int> muteLocalVideoStream(bool mute,
      {int streamType = NERtcVideoStreamType.main}) async {
    apiLogger.i('muteLocalVideoStream#arg{mute:$mute,streamType:$streamType}');
    int reply = await wrapper('muteLocalVideoStream',
        _platform.muteLocalVideoStream(mute, streamType));
    return reply;
  }

  @override
  Future<int> muteLocalSubStreamAudio(bool muted) async {
    apiLogger.i('muteLocalSubStreamAudio#arg{muted:$muted}');
    int reply = await wrapper(
        'muteLocalSubStreamAudio', _platform.muteLocalSubStreamAudio(muted));
    return reply;
  }

  @override
  Future<int> enableLocalSubStreamAudio(bool enable) async {
    apiLogger.i('enableLocalSubStreamAudio#arg{enable:$enable}');
    int reply = await wrapper('enableLocalSubStreamAudio',
        _platform.enableLocalSubStreamAudio(enable));
    return reply;
  }

  @override
  Future<int> setAudioProfile(
      NERtcAudioProfile profile, NERtcAudioScenario scenario) async {
    apiLogger.i('setAudioProfile#arg{profile:$profile,scenario:$scenario}');
    int reply = await wrapper(
        'setAudioProfile', _platform.setAudioProfile(profile, scenario));
    return reply;
  }

  @override
  Future<int> setCameraCaptureConfig(NERtcCameraCaptureConfig captureConfig,
      {int streamType = NERtcVideoStreamType.main}) async {
    apiLogger.i(
        'setCameraCaptureConfig#arg{captureConfig:${captureConfig.toString()},streamType:$streamType}');
    int reply = await wrapper('setCameraCaptureConfig',
        _platform.setCameraCaptureConfig(captureConfig, streamType));
    return reply;
  }

  @override
  Future<int> setLocalVideoConfig(NERtcVideoConfig videoConfig,
      {int streamType = NERtcVideoStreamType.main}) async {
    apiLogger.i(
        'setLocalVideoConfig#arg{videoConfig:${videoConfig.toString()},streamType:$streamType}');
    int reply = await wrapper('setLocalVideoConfig',
        _platform.setLocalVideoConfig(videoConfig, streamType));
    return reply;
  }

  @override
  Future<int> startVideoPreview(
      {int streamType = NERtcVideoStreamType.main}) async {
    apiLogger.i('startVideoPreview#arg{streamType:$streamType}');
    int reply = await wrapper(
        'startVideoPreview', _platform.startVideoPreview(streamType));
    return reply;
  }

  @override
  Future<int> stopVideoPreview(
      {int streamType = NERtcVideoStreamType.main}) async {
    apiLogger.i('stopVideoPreview#arg{streamType:$streamType}');
    int reply = await wrapper(
        'stopVideoPreview', _platform.stopVideoPreview(streamType));
    return reply;
  }

  @override
  Future<int> startScreenCapture(NERtcScreenConfig config) async {
    apiLogger.i('startScreenCapture#arg{config:$config}');
    int reply = await wrapper(
        'startScreenCapture', _platform.startScreenCapture(config));
    return reply;
  }

  @override
  Future<int> stopScreenCapture() async {
    int reply =
        await wrapper('stopScreenCapture', _platform.stopScreenCapture());
    return reply;
  }

  @override
  Future<int> enableLoopbackRecording(bool enable) async {
    apiLogger.i('enableLoopbackRecording#arg{enable:$enable}');
    int reply = await wrapper(
        'enableLoopbackRecording', _platform.enableLoopbackRecording(enable));
    return reply;
  }

  @override
  Future<int> subscribeRemoteSubStreamVideo(int uid, bool subscribe) async {
    apiLogger
        .i('subscribeRemoteSubStreamVideo#arg{uid:$uid,subscribe:$subscribe}');
    int reply = await wrapper('subscribeRemoteSubStreamVideo',
        _platform.subscribeRemoteSubStreamVideo(uid, subscribe));
    return reply;
  }

  @override
  Future<int> subscribeRemoteVideoStream(
      int uid, int streamType, bool subscribe) async {
    apiLogger.i(
        'subscribeRemoteVideoStream#arg{uid:$uid,streamType:$streamType,subscribe:$subscribe}');
    int reply = await wrapper('subscribeRemoteVideoStream',
        _platform.subscribeRemoteVideoStream(uid, streamType, subscribe));
    return reply;
  }

  @override
  Future<int> enableAudioVolumeIndication(bool enable, int interval,
      {bool vad = false}) async {
    apiLogger.i(
        'enableAudioVolumeIndication#arg{enable:$enable,interval:$interval,vad:$vad}');
    int reply = await wrapper('enableAudioVolumeIndication',
        _platform.enableAudioVolumeIndication(enable, interval, vad));
    return reply;
  }

  @override
  Future<int> startAudioDump() async {
    int reply = await wrapper('startAudioDump', _platform.startAudioDump());
    return reply;
  }

  @override
  Future<int> stopAudioDump() async {
    int reply = await wrapper('stopAudioDump', _platform.stopAudioDump());
    return reply;
  }

  @override
  Future<int> enableDualStreamMode(bool enable) async {
    apiLogger.i('enableDualStreamMode#arg{enable:$enable}');
    int reply = await wrapper(
        'enableDualStreamMode', _platform.enableDualStreamMode(enable));
    return reply;
  }

  @override
  Future<int> setChannelProfile(int channelProfile) async {
    apiLogger.i('setChannelProfile#arg{channelProfile:$channelProfile}');
    int reply = await wrapper(
        'setChannelProfile', _platform.setChannelProfile(channelProfile));
    return reply;
  }

  @override
  Future<int> addLiveStreamTask(NERtcLiveStreamTaskInfo taskInfo) async {
    apiLogger.i('addLiveStreamTask#arg{taskInfo:${taskInfo.toString()}}');
    int reply = await wrapper(
        'addLiveStreamTask', _platform.addLiveStreamTask(taskInfo));
    return reply;
  }

  @override
  Future<int> updateLiveStreamTask(NERtcLiveStreamTaskInfo taskInfo) async {
    apiLogger.i('updateLiveStreamTask#arg{taskInfo:${taskInfo.toString()}}');
    int reply = await wrapper(
        'updateLiveStreamTask', _platform.updateLiveStreamTask(taskInfo));
    return reply;
  }

  @override
  Future<int> removeLiveStreamTask(String taskId) async {
    apiLogger.i('removeLiveStreamTask#arg{taskId:$taskId}');
    int reply = await wrapper(
        'removeLiveStreamTask', _platform.removeLiveStreamTask(taskId));
    return reply;
  }

  @override
  Future<int> adjustPlaybackSignalVolume(int volume) async {
    apiLogger.i('adjustPlaybackSignalVolume#arg{volume:$volume}');
    int reply = await wrapper('adjustPlaybackSignalVolume',
        _platform.adjustPlaybackSignalVolume(volume));
    return reply;
  }

  @override
  Future<int> adjustRecordingSignalVolume(int volume) async {
    apiLogger.i('adjustRecordingSignalVolume#arg{volume:$volume}');
    int reply = await wrapper('adjustRecordingSignalVolume',
        _platform.adjustRecordingSignalVolume(volume));
    return reply;
  }

  @override
  Future<int> adjustLoopBackRecordingSignalVolume(int volume) async {
    apiLogger.i('adjustLoopBackRecordingSignalVolume#arg{volume:$volume}');
    int reply = await wrapper('adjustLoopBackRecordingSignalVolume',
        _platform.adjustLoopBackRecordingSignalVolume(volume));
    return reply;
  }

  @override
  Future<int> getConnectionState() async {
    int reply =
        await wrapper('getConnectionState', _platform.getConnectionState());
    return reply;
  }

  @override
  Future<int> sendSEIMsg(String seiMsg,
      {int streamType = NERtcVideoStreamType.main}) async {
    apiLogger.i('sendSEIMsg#arg{seiMsg:$seiMsg,streamType:$streamType}');
    int reply =
        await wrapper('sendSEIMsg', _platform.sendSEIMsg(seiMsg, streamType));
    return reply;
  }

  @override
  Future<int> setClientRole(int role) async {
    apiLogger.i('setClientRole#arg{role:$role}');
    int reply = await wrapper('setClientRole', _platform.setClientRole(role));
    return reply;
  }

  @override
  Future<int> uploadSdkInfo() async {
    int reply = await wrapper('uploadSdkInfo', _platform.uploadSdkInfo());
    return reply;
  }

  @override
  Future<int> setAudioEffectPreset(int preset) async {
    apiLogger.i('setAudioEffectPreset#arg{preset:$preset}');
    int reply = await wrapper(
        'setAudioEffectPreset', _platform.setAudioEffectPreset(preset));
    return reply;
  }

  @override
  Future<int> setLocalVoiceEqualization(int bandFrequency, int bandGain) async {
    apiLogger.i(
        'setLocalVoiceEqualization#arg{bandFrequency:$bandFrequency,bandGain:$bandGain}');
    int reply = await wrapper('setLocalVoiceEqualization',
        _platform.setLocalVoiceEqualization(bandFrequency, bandGain));
    return reply;
  }

  @override
  Future<int> setLocalVoicePitch(double pitch) async {
    apiLogger.i('setLocalVoicePitch#arg{pitch:$pitch}');
    int reply = await wrapper(
        'setLocalVoicePitch', _platform.setLocalVoicePitch(pitch));
    return reply;
  }

  @override
  Future<int> setVoiceBeautifierPreset(int preset) async {
    apiLogger.i('setVoiceBeautifierPreset#arg{preset:$preset}');
    int reply = await wrapper(
        'setVoiceBeautifierPreset', _platform.setVoiceBeautifierPreset(preset));
    return reply;
  }

  @override
  Future<int> setLocalVoiceReverbParam(NERtcReverbParam param) async {
    apiLogger.i('setLocalVoiceReverbParam#arg{param:${param.toString()}}');
    int reply = await wrapper(
        'setLocalVoiceReverbParam', _platform.setLocalVoiceReverbParam(param));
    return reply;
  }

  @override
  Future<int> startAudioRecording(
      String filePath, int sampleRate, int quality) async {
    apiLogger.i(
        'startAudioRecording#arg{filePath:$filePath,sampleRate:$sampleRate,quality:$quality}');
    int reply = await wrapper('startAudioRecording',
        _platform.startAudioRecording(filePath, sampleRate, quality));
    return reply;
  }

  @override
  Future<int> startAudioRecordingWithConfig(
      NERtcAudioRecordingConfiguration config) async {
    apiLogger
        .i('startAudioRecordingWithConfig#arg{config:${config.toString()}}');
    int reply = await wrapper('startAudioRecordingWithConfig',
        _platform.startAudioRecordingWithConfig(config));
    return reply;
  }

  @override
  Future<int> stopAudioRecording() async {
    int reply =
        await wrapper('stopAudioRecording', _platform.stopAudioRecording());
    return reply;
  }

  @override
  Future<int> switchChannel(String? token, String channelName,
      {NERtcJoinChannelOptions? channelOptions}) async {
    apiLogger.i(
        'switchChannel#arg{token:$token,channelName:$channelName,channelOptions:${channelOptions.toString()}}');
    int reply = await wrapper(
        'switchChannel',
        _platform.switchChannel(token, channelName,
            channelOptions: channelOptions ?? null));
    return reply;
  }

  @override
  Future<int> setLocalMediaPriority(int priority, bool isPreemptive) async {
    apiLogger.i(
        'setLocalMediaPriority#arg{priority:$priority,isPreemptive:$isPreemptive}');
    int reply = await wrapper('setLocalMediaPriority',
        _platform.setLocalMediaPriority(priority, isPreemptive));
    return reply;
  }

  @override
  Future<int> startChannelMediaRelay(
      NERtcChannelMediaRelayConfiguration config) async {
    apiLogger.i('startChannelMediaRelay#arg{config:${config.toString()}}');
    int reply = await wrapper(
        'startChannelMediaRelay', _platform.startChannelMediaRelay(config));
    return reply;
  }

  @override
  Future<int> stopChannelMediaRelay() async {
    int reply = await wrapper(
        'stopChannelMediaRelay', _platform.stopChannelMediaRelay());
    return reply;
  }

  @override
  Future<int> updateChannelMediaRelay(
      NERtcChannelMediaRelayConfiguration config) async {
    apiLogger.i('updateChannelMediaRelay#arg{config:${config.toString()}}');
    int reply = await wrapper(
        'updateChannelMediaRelay', _platform.updateChannelMediaRelay(config));
    return reply;
  }

  @override
  Future<int> adjustUserPlaybackSignalVolume(int uid, int volume) async {
    apiLogger.i('adjustUserPlaybackSignalVolume#arg{uid:$uid,volume:$volume}');
    int reply = await wrapper('adjustUserPlaybackSignalVolume',
        _platform.adjustUserPlaybackSignalVolume(uid, volume));
    return reply;
  }

  @override
  Future<int> reportCustomEvent(String eventName, String? customIdentify,
      Map<String?, Object?>? param) async {
    apiLogger.i(
        'reportCustomEvent#arg{eventName:$eventName,customIdentify:$customIdentify,param:$param}');
    int reply = await wrapper('reportCustomEvent',
        _platform.reportCustomEvent(eventName, customIdentify, param));
    return reply;
  }

  @override
  Future<int> enableEncryption(
      bool enable, NERtcEncryptionConfig config) async {
    apiLogger
        .i('enableEncryption#arg{enable:$enable,config:${config.toString()}}');
    int reply = await wrapper(
        'enableEncryption', _platform.enableEncryption(enable, config));
    return reply;
  }

  @override
  Future<int> enableSuperResolution(bool enable) async {
    apiLogger.i('enableSuperResolution#arg{enable:$enable}');
    int reply = await wrapper(
        'enableSuperResolution', _platform.enableSuperResolution(enable));
    return reply;
  }

  @override
  Future<int> setAudioSessionOperationRestriction(
      NERtcAudioSessionOperationRestriction restriction) async {
    apiLogger
        .i('setAudioSessionOperationRestriction#arg{restriction:$restriction}');
    int reply = await wrapper('setAudioSessionOperationRestriction',
        _platform.setAudioSessionOperationRestriction(restriction));
    return reply;
  }

  @override
  Future<int> setLocalPublishFallbackOption(int option) async {
    apiLogger.i('setLocalPublishFallbackOption#arg{option:$option}');
    int reply = await wrapper('setLocalPublishFallbackOption',
        _platform.setLocalPublishFallbackOption(option));
    return reply;
  }

  @override
  Future<int> setRemoteSubscribeFallbackOption(int option) async {
    apiLogger.i('setRemoteSubscribeFallbackOption#arg{option:$option}');
    int reply = await wrapper('setRemoteSubscribeFallbackOption',
        _platform.setRemoteSubscribeFallbackOption(option));
    return reply;
  }

  @override
  Future<int> getEffectDuration(int effectId) async {
    apiLogger.i('getEffectDuration#arg{effectId:$effectId}');
    int reply = await wrapper(
        'getEffectDuration', _platform.getEffectDuration(effectId));
    return reply;
  }

  @override
  Future<int> startLastmileProbeTest(LastmileProbeConfig config) async {
    apiLogger.i('startLastmileProbeTest#arg{config:$config}');
    int reply = await wrapper(
        'startLastmileProbeTest', _platform.startLastmileProbeTest(config));
    return reply;
  }

  @override
  Future<int> stopLastmileProbeTest() async {
    int reply = await wrapper(
        'stopLastmileProbeTest', _platform.stopLastmileProbeTest());
    return reply;
  }

  @override
  Future<int> setVideoCorrectionConfig(
      NERtcVideoCorrectionConfiguration? config) async {
    apiLogger.i('setVideoCorrectionConfig#arg{config:$config}');
    int reply = await wrapper(
        'setVideoCorrectionConfig', _platform.setVideoCorrectionConfig(config));
    return reply;
  }

  @override
  Future<int> enableVirtualBackground(
      bool enabled, NERtcVirtualBackgroundSource? backgroundSource) async {
    apiLogger.i(
        'enableVirtualBackground#arg{enabled:$enabled,backgroundSource:$backgroundSource}');
    int reply = await wrapper('enableVirtualBackground',
        _platform.enableVirtualBackground(enabled, backgroundSource));
    return reply;
  }

  @override
  Future<int> setCloudProxy(int proxyType) async {
    apiLogger.i('setCloudProxy#arg{proxyType:$proxyType}');
    int reply =
        await wrapper('setCloudProxy', _platform.setCloudProxy(proxyType));
    return reply;
  }

  @override
  Future<int> setRemoteHighPriorityAudioStream(bool enabled, int uid,
      {int streamType = NERtcAudioStreamType.kNERtcAudioStreamTypeMain}) async {
    apiLogger.i(
        'setRemoteHighPriorityAudioStream#arg{enabled:$enabled,uid:$uid,streamType:$streamType}');
    int reply = await wrapper('setRemoteHighPriorityAudioStream',
        _platform.setRemoteHighPriorityAudioStream(enabled, uid, streamType));
    return reply;
  }

  @override
  Future<int> startBeauty() async {
    int reply = await wrapper('startBeauty', _platform.startBeauty());
    return reply;
  }

  @override
  Future<void> stopBeauty() async {
    await wrapper('stopBeauty', _platform.stopBeauty());
    return;
  }

  @override
  Future<int> enableBeauty(bool enabled) async {
    apiLogger.i('enableBeauty#arg{enabled:$enabled}');
    int reply = await wrapper('enableBeauty', _platform.enableBeauty(enabled));
    return reply;
  }

  @override
  Future<int> setBeautyEffect(double level, int beautyType) async {
    apiLogger.i('setBeautyEffect#arg{level:$level,beautyType:$beautyType}');
    int reply = await wrapper(
        'setBeautyEffect', _platform.setBeautyEffect(level, beautyType));
    return reply;
  }

  @override
  Future<int> addBeautyFilter(String path, {String name = ''}) async {
    apiLogger.i('addBeautyFilter#arg{path:$path,name:$name}');
    int reply =
        await wrapper('addBeautyFilter', _platform.addBeautyFilter(path, name));
    return reply;
  }

  @override
  Future<void> removeBeautyFilter() async {
    await wrapper('removeBeautyFilter', _platform.removeBeautyFilter());
    return;
  }

  @override
  Future<int> setBeautyFilterLevel(double level) async {
    apiLogger.i('setBeautyFilterLevel#arg{level:$level}');
    int reply = await wrapper(
        'setBeautyFilterLevel', _platform.setBeautyFilterLevel(level));
    return reply;
  }

  @override
  Future<int> setLocalVideoWatermarkConfigs(
      int type, NERtcVideoWatermarkConfig? config) async {
    apiLogger.i('setLocalVideoWatermarkConfigs#arg{type:$type,config:$config}');
    int reply = await wrapper('setLocalVideoWatermarkConfigs',
        _platform.setLocalVideoWatermarkConfigs(type, config));
    return reply;
  }

  @override
  Future<int> startAudioDumpWithType(int dumpType) async {
    apiLogger.i('startAudioDumpWithType#arg{dumpType:$dumpType}');
    int reply = await wrapper(
        'startAudioDumpWithType', _platform.startAudioDumpWithType(dumpType));
    return reply;
  }

  @override
  Future<int> getNtpTimeOffset() async {
    int reply = await wrapper('getNtpTimeOffset', _platform.getNtpTimeOffset());
    return reply;
  }

  @override
  Future<int> setStreamAlignmentProperty(bool enable) async {
    apiLogger.i('setStreamAlignmentProperty#arg{enable:$enable}');
    int reply = await wrapper('setStreamAlignmentProperty',
        _platform.setStreamAlignmentProperty(enable));
    return reply;
  }

  @override
  void onAudioEffectFinished(int effectId) {
    if (!_initialized) return;
    commonLogger.i('onAudioEffectFinished: effectId: $effectId');
    _audioEffectMgr.audioEffectEventCallbacks.forEach((element) {
      element.onAudioEffectFinished(effectId);
    });
  }

  @override
  void onAudioEffectTimestampUpdate(int id, int timestampMs) {
    if (!_initialized) return;
    commonLogger
        .i('onAudioEffectTimestampUpdate: id: $id, timestampMs: $timestampMs');
    _audioEffectMgr.audioEffectEventCallbacks.forEach((element) {
      element.onAudioEffectTimestampUpdate(id, timestampMs);
    });
  }

  @override
  void onAudioMixingStateChanged(int reason) {
    if (!_initialized) return;
    commonLogger.i('onAudioMixingStateChanged: reason: $reason');
    _audioMixingMgr.audioMixingEventCallbacks.forEach((element) {
      element.onAudioMixingStateChanged(reason);
    });
  }

  @override
  void onAudioMixingTimestampUpdate(int timestampMs) {
    if (!_initialized) return;
    _audioMixingMgr.audioMixingEventCallbacks.forEach((element) {
      element.onAudioMixingTimestampUpdate(timestampMs);
    });
  }

  @override
  void onLocalAudioStats(Map<dynamic, dynamic> arguments) {
    if (!_initialized) return;
    for (var callback in _statsCallbacks.copy()) {
      callback.onLocalAudioStats(NERtcAudioSendStats.fromMap(arguments));
    }
  }

  @override
  void onLocalVideoStats(Map<dynamic, dynamic> arguments) {
    if (!_initialized) return;
    for (var callback in _statsCallbacks.copy()) {
      callback.onLocalVideoStats(NERtcVideoSendStats.fromMap(arguments));
    }
  }

  @override
  void onNetworkQuality(Map<dynamic, dynamic> arguments) {
    if (!_initialized) return;
    List<dynamic> argumentList = arguments['list'];
    var statsList = <NERtcNetworkQualityInfo>[];
    for (Map<dynamic, dynamic> argument in argumentList) {
      statsList.add(NERtcNetworkQualityInfo.fromMap(argument));
    }
    for (var callback in _statsCallbacks.copy()) {
      callback.onNetworkQuality(statsList);
    }
  }

  @override
  void onRemoteAudioStats(Map<dynamic, dynamic> arguments) {
    if (!_initialized) return;
    List<dynamic> argumentList = arguments['list'];
    List<NERtcAudioRecvStats> statsList = <NERtcAudioRecvStats>[];
    for (Map<dynamic, dynamic> argument in argumentList) {
      statsList.add(NERtcAudioRecvStats.fromMap(argument));
    }
    for (var callback in _statsCallbacks.copy()) {
      callback.onRemoteAudioStats(statsList);
    }
  }

  @override
  void onRemoteVideoStats(Map<dynamic, dynamic> arguments) {
    if (!_initialized) return;
    List<dynamic> argumentList = arguments['list'];
    var statsList = <NERtcVideoRecvStats>[];
    for (Map<dynamic, dynamic> argument in argumentList) {
      statsList.add(NERtcVideoRecvStats.fromMap(argument));
    }
    for (var callback in _statsCallbacks.copy()) {
      callback.onRemoteVideoStats(statsList);
    }
  }

  @override
  void onRtcStats(Map<dynamic, dynamic> arguments) {
    if (!_initialized) return;
    for (var callback in _statsCallbacks.copy()) {
      callback.onRtcStats(NERtcStats.fromMap(arguments));
    }
  }

  @override
  void onLastmileProbeResult(NERtcLastmileProbeResult result) {
    commonLogger.i('onLastmileProbeResult: $result');
    for (var callback in _eventCallbacks.copy()) {
      callback.onLastmileProbeResult(result);
    }
  }

  @override
  void onLastmileQuality(int quality) {
    commonLogger.i('onLastmileQuality: $quality');
    for (var callback in _eventCallbacks.copy()) {
      callback.onLastmileQuality(quality);
    }
  }

  @override
  Future<int> enableMediaPub(int mediaType, bool enable) async {
    apiLogger.i('enableMediaPub#arg{enable:$enable,mediaType:$mediaType}');
    int reply = await wrapper(
        'enableMediaPub', _platform.enableMediaPub(mediaType, enable));
    return reply;
  }

  @override
  void onMediaRightChange(
      bool isAudioBannedByServer, bool isVideoBannedByServer) {
    commonLogger.i(
        'onMediaRightChange: isAudioBannedByServer$isAudioBannedByServer,isVideoBannedByServer:$isVideoBannedByServer');
    for (var callback in _eventCallbacks.copy()) {
      callback.onMediaRightChange(isAudioBannedByServer, isVideoBannedByServer);
    }
  }

  @override
  Future<int> setAudioSubscribeOnlyBy(List<int>? uidArray) async {
    apiLogger.i('setAudioSubscribeOnlyBy#arg{uidArray:${uidArray?.join(',')}}');
    int reply = await wrapper(
        'setAudioSubscribeOnlyBy', _platform.setAudioSubscribeOnlyBy(uidArray));
    return reply;
  }

  @override
  Future<int> subscribeRemoteSubStreamAudio(int uid, bool subscribe) async {
    apiLogger
        .i('subscribeRemoteSubStreamAudio#arg{uid:$uid,subscribe:$subscribe}');
    int reply = await wrapper('subscribeRemoteSubStreamAudio',
        _platform.subscribeRemoteSubStreamAudio(uid, subscribe));
    return reply;
  }

  @override
  void onTakeSnapshotResult(int code, String path) {
    commonLogger.i('onTakeSnapshotResult: code: $code,path:$path');
    for (var callback in _eventCallbacks.copy()) {
      callback.onTakeSnapshotResult(code, path);
    }
  }

  @override
  Future<int> takeLocalSnapshot(int streamType, String path) async {
    apiLogger.i('takeLocalSnapshot#arg{streamType:$streamType,path:$path}');
    int reply = await wrapper(
        'takeLocalSnapshot', _platform.takeLocalSnapshot(streamType, path));
    return reply;
  }

  @override
  Future<int> takeRemoteSnapshot(int uid, int streamType, String path) async {
    apiLogger.i(
        'takeRemoteSnapshot#arg{uid:$uid,streamType:$streamType,path:$path}');
    int reply = await wrapper('takeRemoteSnapshot',
        _platform.takeRemoteSnapshot(uid, streamType, path));
    return reply;
  }

  @override
  void onAddLiveStreamTask(String taskId, int errCode) {
    commonLogger.i('onAddLiveStreamTask: taskId: $taskId,errCode:$errCode');
    for (var callback in _liveTaskCallback.copy()) {
      callback.onAddLiveStreamTask(taskId, errCode);
    }
  }

  @override
  void onDeleteLiveStreamTask(String taskId, int errCode) {
    commonLogger.i('onDeleteLiveStreamTask: taskId: $taskId,errCode:$errCode');
    for (var callback in _liveTaskCallback.copy()) {
      callback.onDeleteLiveStreamTask(taskId, errCode);
    }
  }

  @override
  void onUpdateLiveStreamTask(String taskId, int errCode) {
    commonLogger.i('onUpdateLiveStreamTask: taskId: $taskId,errCode:$errCode');
    for (var callback in _liveTaskCallback.copy()) {
      callback.onUpdateLiveStreamTask(taskId, errCode);
    }
  }

  @override
  Future<int> enableVideoCorrection(bool enable) async {
    apiLogger.i('enableVideoCorrection#arg{enable:$enable}');
    int reply = await wrapper(
        'enableVideoCorrection', _platform.enableVideoCorrection(enable));
    return reply;
  }
}

extension IterableX<E> on Iterable<E> {
  Iterable<E> copy() {
    return List<E>.from(this, growable: false);
  }

  E? get firstOrNull {
    return isEmpty ? null : first;
  }

  E? get lastOrNull {
    return isEmpty ? null : last;
  }
}
