// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nertc;

/// Use this widget to ensure that the renderer is created only once.
/// And the renderer will be disposed when the widget is unmounted.
/// This make sure that one creation is corresponding to one dispose without leak.
class NERtcVideoView extends StatefulWidget {
  /// The renderer to be used.
  /// If not provided, a new renderer will be created and manage its lifecycle inside.
  /// If provided, renderer owners should manage its lifecycle self.
  final NERtcVideoRenderer? renderer;

  final NERtcVideoRendererEventListener? rendererEventLister;

  /// target user id, or null for local video
  final int? uid;

  /// video is substream or not
  final bool subStream;

  /// auto attach renderer to video.
  /// if renderer is not provided, this should be true.
  final bool autoAttach;

  /// video mirror or not
  final ValueListenable<bool>? mirrorListenable;

  /// video fit type
  final NERtcVideoViewFitType fitType;

  /// video background color
  final Color backgroundColor;

  /// placeholder widget build before renderer available
  final WidgetBuilder? placeholderBuilder;

  const NERtcVideoView({
    Key? key,
    this.uid,
    this.subStream = false,
    this.autoAttach = true,
    this.mirrorListenable,
    this.renderer,
    this.rendererEventLister,
    this.fitType = NERtcVideoViewFitType.cover,
    this.placeholderBuilder,
    this.backgroundColor = const Color(0xFF292933),
  })  : assert(renderer != null || autoAttach),
        super(key: key);

  @override
  State<NERtcVideoView> createState() => NERtcVideoViewState();
}

class NERtcVideoViewState extends State<NERtcVideoView>
    with NERtcVideoRendererEventListener {
  NERtcVideoRenderer? _internalRenderer;
  var _creating = false;

  NERtcVideoRenderer get renderer => widget.renderer ?? _internalRenderer!;

  bool get hasRenderer => widget.renderer != null || _internalRenderer != null;

  void _initRenderer() async {
    if (hasRenderer || _creating) {
      return;
    }
    _creating = true;
    if (widget.renderer == null) {
      _internalRenderer = await NERtcVideoRendererFactory.createVideoRenderer();
      _internalRenderer!.rendererEventLister = widget.rendererEventLister;
    }
    if (mounted) {
      setState(() {
        _attachRender();
      });
    } else {
      _disposeRenderer();
    }
    _creating = false;
  }

  void _disposeRenderer() {
    _internalRenderer?.rendererEventLister = null;
    _internalRenderer?.dispose();
    _internalRenderer = null;
  }

  void _attachRender() async {
    if (widget.autoAttach) {
      final uid = widget.uid;
      final subStream = widget.subStream;
      int result = -1;
      if (uid == null) {
        result = await (!subStream
            ? renderer.attachToLocalVideo()
            : renderer.attachToLocalSubStreamVideo());
      } else {
        result = await (!subStream
            ? renderer.attachToRemoteVideo(uid)
            : renderer.attachToRemoteSubStreamVideo(uid));
      }
      debugPrint('attach render to $uid#$subStream result: $result');
    }
    _updateMirror();
  }

  void _updateMirror() {
    if (hasRenderer && widget.mirrorListenable != null) {
      renderer.setMirror(widget.mirrorListenable!.value);
    }
  }

  @override
  void initState() {
    super.initState();
    _initRenderer();
    widget.mirrorListenable?.addListener(_updateMirror);
  }

  @override
  void dispose() {
    _disposeRenderer();
    super.dispose();
  }

  @override
  void didUpdateWidget(NERtcVideoView oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.uid != oldWidget.uid ||
        widget.subStream != oldWidget.subStream ||
        widget.renderer != oldWidget.renderer) {
      _disposeRenderer();
      _initRenderer();
    }
    if (widget.mirrorListenable != oldWidget.mirrorListenable) {
      oldWidget.mirrorListenable?.removeListener(_updateMirror);
      widget.mirrorListenable?.addListener(_updateMirror);
    }
    _updateMirror();
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      print(
          'build video view: ${constraints.maxWidth}x${constraints.maxHeight}');
      return Container(
        width: constraints.maxWidth,
        height: constraints.maxHeight,
        color: widget.backgroundColor,
        // alignment: Alignment.center,
        child: hasRenderer
            ? ValueListenableBuilder<_NERtcVideoValue>(
                valueListenable: renderer,
                builder: (BuildContext context, _NERtcVideoValue value,
                    Widget? child) {
                  return renderer.canRender && renderer.textureId != null
                      ? _buildVideoView(
                          renderer.textureId!,
                          constraints.maxHeight * value.aspectRatio,
                          constraints.maxHeight,
                        )
                      : _buildPlaceHolder();
                },
              )
            : _buildPlaceHolder(),
      );
    });
  }

  Widget _buildPlaceHolder() {
    return widget.placeholderBuilder?.call(context) ??
        Container(
          color: widget.backgroundColor,
        );
  }

  Widget _buildVideoView(int textureId, double width, double height) {
    return FittedBox(
      fit: widget.fitType == NERtcVideoViewFitType.contain
          ? BoxFit.contain
          : BoxFit.cover,
      child: Center(
        child: SizedBox(
            height: height, width: width, child: Texture(textureId: textureId)),
      ),
    );
  }

  @override
  void onFirstFrameRendered(int uid) {
    Alog.i(
      tag: 'RoomUserVideoViewState',
      content: 'onFirstFrameRendered for $uid  ${widget.subStream}',
    );
    widget.rendererEventLister?.onFirstFrameRendered(uid);
  }

  @override
  void onFrameResolutionChanged(int uid, int width, int height, int rotation) {
    Alog.i(
      tag: 'RoomUserVideoViewState',
      content: 'onFrameResolutionChanged for $uid ${widget.subStream}',
    );
    widget.rendererEventLister
        ?.onFrameResolutionChanged(uid, width, height, rotation);
  }
}
